function [p, pc] = orthopoly(type, n)
% Compute coefficients for the orthogonal polynomials up to degree N

p = zeros(n+1, n+1);    % in matrix form
pc = cell(n+1, 1);      % in cell array form

switch type,
    case 'legendre',    % generate the Legendre polynomials
        pc{1} = 1;
        pc{2} = [1 0];
        for i = 2:n,
            d = i-1;    % degree of last polynomial
            pc{i+1} = (2*d+1)/(d+1) * conv([1 0], pc{i}) - d/(d+1) * [0 0 pc{i-1}];
        end;
        
    case 'cheby',       % generate the Chebyshev polynomials of the first kind
        pc{1} = 1;
        pc{2} = [1 0];
        for i = 2:n, 
            pc{i+1} = 2 * conv([1 0], pc{i}) - [0 0 pc{i-1}];
        end;
        
    case 'cheby2',       % generate the Chebyshev polynomials of the second kind
        pc{1} = 1;
        pc{2} = [2 0];
        for i = 2:n,
            pc{i+1} = 2 * conv([1 0], pc{i}) - [0 0 pc{i-1}];
        end;
        
    otherwise,
        error(['Orthogonal polynomial type [' type '] unknown']);
end;

% use the cell-array form to compute the matrix form
for i = 0:n,
    p(i+1, (n-i+1):(n+1)) = pc{i+1};
end;

% END orthopoly RETURNING p, pc