function rd = rbfquad(model, cfg)
% Create RBF state approximator with quadratic approximation in the action space
% Requires single-dimensional action space. Does not support truncation of RBFs.
% This approximator type is now obsolete. RBFPOLY should be used instead with M=2 (order 2, quadratic)

CFG.type = 'rbfquad';            % # of samples to generate
CFG.disc = 0;
CFG.c = [];
CFG.rad = [];

cfg = parseconfig(cfg, CFG);

if isempty(cfg.c) || isempty(cfg.rad),
    error('Centers and radii of RBFs could not be obtained');
end;

rd.type = cfg.type;
rd.sparse = 0;       % recommend usage of sparse activation vectors
% core functions
rd.phi = @rbfquad_phi; rd.q = @rbfquad_q; rd.h = @rbfquad_h; 
% plot functions
rd.plotv = @rbfquad_plotv; rd.ploth = @rbfquad_ploth;
% centers & radii
rd.c = cfg.c;
rd.rad = cfg.rad;
rd.Nc = size(rd.c, 2);
if (size(rd.c, 1) ~= model.p) || any(size(rd.c) ~= size(rd.rad)),
    error('Incorrect c or rad size');
end;
% total number of basis functions
rd.N = rd.Nc * 3;
rd.uint = [-model.maxu 0 model.maxu];    % this should be changed for non-symmetric U

end     % TRIANG constructor


% Compute BF values in x, u
function phi = rbfquad_phi(rd, x, u)
phi = nrbf(x, rd.Nc, rd.c, rd.rad)';
phi = [phi; phi.*u; phi.*(u*u)];
end

% Compute Q-value in x, u
function q = rbfquad_q(rd, theta, x, u)
phi = nrbf(x, rd.Nc, rd.c, rd.rad);
q = [phi phi.*u phi.*(u*u)] * theta;
end

% Compute greedy policy in x
function u = rbfquad_h(rd, theta, x)
phi = nrbf(x, rd.Nc, rd.c, rd.rad);
% a = mu * theta(ind);      - the u^0 (bias) coefficient is irrelevant here
b = phi * theta(rd.Nc+1:2*rd.Nc);
c = phi * theta(2*rd.Nc+1:end);
if c < 0,           % function is concave, the max is in u0 -- but make sure it's within the interval
    u0 = -b/2/c;
    u = max(min(u0, rd.uint(3)), rd.uint(1));
elseif c > 0,       % function is convex, max either at start or end of u interval
    u0 = -b/2/c;
    if u0 < rd.uint(2),
        % minimum in the left part of the interval, maximum is therefore at the right end of the
        % interval
        u = rd.uint(3);
    else% the other way around
        u = rd.uint(1);
    end;
else                % c = 0
    if b > 0,       % linear approximation with positive slope, max at right
        u = rd.uint(3);
    else            % the other way around
        u = rd.uint(1);
    end;
end;
end


% ----- Plot functions
% Plot V - forward call to generic plot fun
function rbfquad_plotv(r, theta, varargin)
for i = size(r.c, 1):-1:1, xb(i, 1:2) = [min(r.c(i, :)) max(r.c(i, :))]; end;
ub = r.uint([1 end]);
approx_plotv(r, theta, xb, ub, varargin{:});
end
% Plot h - forward call to generic plot fun
function rbfquad_ploth(r, theta, varargin)
for i = size(r.c, 1):-1:1, xb(i, 1:2) = [min(r.c(i, :)) max(r.c(i, :))]; end;
ub = r.uint([1 end]);
approx_ploth(r, theta, xb, ub, varargin{:});
end
