function id = generateexpid(cfg, params, algcfg, rep)
% Experiment ID structure:
% prefix_alg_problem_infix_par1val1_...parNvalN_postfix_#repetitionindex
% where prefix, infix, postfix are optional

id = '';

% apply any prefix
if ~isempty(cfg.idprefix), id = [id cfg.idprefix '_']; end;

% algorithm name: special treatment -- try getting short ID
if isempty(cfg.idfields) || any(strcmp(cfg.idfields, 'alg')),
    algid = feval([cfg.alg '_exp'], 'shortid'); 
    if isempty(algid), algid = cfg.alg; end;
    id = [id algid];
end;

% problem name: special treatment -- try getting short ID
if isfield(params, 'problem') && (isempty(cfg.idfields) || any(strcmp(cfg.idfields, 'problem'))),
    probid = feval(algcfg.problem, 'shortid'); 
    if isempty(probid), probid = algcfg.problem; end;
    id = [id '_' probid];
end;

% apply any infix
if ~isempty(cfg.idinfix), id = [id '_' cfg.idinfix]; end;

% ordinary parameters now
pids = fieldnames(params);
for i = 1:length(pids),
    pid = pids{i};
    if strcmp(pid, 'problem'), continue; end;   % already processed
    % skip the parameter if:
    %   - it is a trivial parameter (e.g., display options etc.)
    %   - IDFIELDS is set and this parameter is not there
    if ~isempty(cfg.idfields) && ~any(strcmp(cfg.idfields, pid)), continue; end;
    p = params.(pid);
    val = eval(['algcfg.' p.path]);
    [fid, fval] = formatparam(p, val);
    if isempty(fid) && isempty(fval), continue; end;     % parameter cannot be formatted
    id = [id sprintf('_%s%s', fid, fval)];
end;

% apply any postfix
if ~isempty(cfg.idpostfix), id = [id '_' cfg.idpostfix]; end;

% repetition index: special treatment -- only show if more than 1 repetitions OR explicitly on idfields
show = cfg.repeat > 1 || (~isempty(cfg.idfields) && any(strcmp(cfg.idfields, 'rep')));
if show,
    id = [id sprintf('_#%02d', rep)];
end;
end     % function GEN_DFNAME