function varargout = qlearn_exp(mode, cfg)
% Handle experiment in batch run, for classical Q-learning

% TODO implement cleanup

if nargin < 2, cfg = struct; end;

switch mode,
    % provide short ID
    case 'shortid',
        varargout = {'qlearn'};

    % provide configuration parameters
    case 'params',
        p = struct;
        % main parameters
        p.problem = [];
        p.gamma = [];
        p.alpha = []; p.alphadecay = []; p.explor = []; p.explordecay = []; p.lambda = [];
        p.maxtrials = []; p.maxsteps = []; 
        p.ertype.path = 'ertype'; p.reset.type = 'string'; p.reset.shortid = 'er';
        p.erN = [];
        p.reset.path = 'reset'; p.reset.type = 'array'; p.reset.shortid = 'res';
        % remaining, "trivial" parameters
        p = addtrivialparams(p, 'eps', 'model_params', 'reset', 'datadir', 'verb', 'sampledisp', 'trialdisp');
        varargout = {p};
        
    % provide input variables
    case 'inputs',
        in = struct;
        % distance to Qstar, requires:
        %   Qstar on config
        %   Qh for all experiments
        %   that all experiments have the same #trials, equal to maxtrial
        in.trial.type = 'scalar';
        in.trial.id = 'trial';
        in.trial.label = 'trial number';
        in.trial.val = [];
        
        varargout = {in};
        
    % provide output variables
    case 'outputs',
        o = struct;
        % distance to Qstar, requires:
        %   Qstar on config
        %   Qh for all experiments
        %   that all experiments have the same #trials, equal to maxtrial
        o.distqstar.id = 'distqstar';
        o.distqstar.label = 'distance to Q*';
        o.distqstar.params = {'Qstar'};
        
        o.disthstar.id = 'disthstar';
        o.disthstar.label = 'distance to h*';
        o.disthstar.params = {'Qstar', 'hstar'};
        
        o.exectime.id = 'exectime';
        o.exectime.label = 'execution time';
        o.exectime.units = 's';
        o.exectime.scale = 'log';
        
        o.onlret.id = 'onlret';
        o.onlret.label = 'online return';
        o.onlret.params = {};
       
        varargout = {o};
        
    % process options
    case {'run'},
        cfg.(mode) = 1;
        qlearn(cfg);
        
    % compute execution time
    case 'exectime',
        load(cfg.datafile, 'timestat');
        varargout = {timestat.run};
        
    % evaluate regret: note this is an 'inner'-variable dependent output, so it must also return the
    % input values
    case 'distqstar',
        % load experiment data
        loadwithprefix(cfg.datafile, 'e_', {'cfg', 'Qh'});
        dq = zeros(e_cfg.maxtrials, 1);
        for trial = 1:e_cfg.maxtrials,
            % March 17: use 2-norm, it's more informative
            dq(trial) = norm(e_Qh{trial+1}(:) - cfg.Qstar(:), 2);
            % dq(trial) = max(abs(e_Qh{trial+1}(:) - cfg.Qstar(:)));
        end;
        varargout = {1:e_cfg.maxtrials, dq};
        
    % evaluate regret: note this is an 'inner'-variable dependent output, so it must also return the
    % input values
    case 'disthstar',
        % load experiment data
        loadwithprefix(cfg.datafile, 'e_', {'cfg', 'Qh', 'Uflat', 'model'});
        dh = zeros(e_cfg.maxtrials, 1);
        Qstar = cfg.Qstar; inonterm = find(~e_model.Xterminal);
        for trial = 1:e_cfg.maxtrials,
            % count the states where the first greedy action in Q is suboptimal
            dh(trial) = 0;
            Q = e_Qh{trial+1};
            for i = inonterm,
                [qmax jmax] = max(Q(i, :));
                if Qstar(i, jmax) < max(Qstar(i, :)), dh(trial) = dh(trial) + 1; end;
            end;
        end;
        varargout = {1:e_cfg.maxtrials, dh};
        
    % evaluate regret: note this is an 'inner'-variable dependent output, so it must also return the
    % input values
    case 'onlret',
        % load experiment data
        loadwithprefix(cfg.datafile, 'e_', {'cfg', 'R'});
        ret = zeros(e_cfg.maxtrials, 1);
        for trial = 1:e_cfg.maxtrials,
            K = find(~isnan(e_R(1, :, trial)), 1, 'last');
            if K == 1, K = e_cfg.maxsteps+1; end;
            ret(trial) = sum((e_cfg.gamma .^ (0:K-2)) .* e_R(1, 2:K, trial));
        end;
        varargout = {1:e_cfg.maxtrials, ret};
        
    otherwise,  % return empty
        varargout = {[]};
end;

end     % main function genmpc_exp

% Local function for lookup-policy (used in the regret branch above)
function u = lookuph(x, X, U)
    i = findflat(x, X, 1, 'first');
    if isempty(i), error('State not part of the set considered by GENMPC'); end;
    u = U(:, i);
end
