function plotslice(fun, xb, cfg)
% Generic slice-plot for n-dimensional functions
CFG.npoints = 30;
CFG.slice = [];     % default slice will be [NaN, NaN, 0, 0, ..., 0]
if nargin < 3, cfg = []; end;
cfg = parseconfig(cfg, CFG);

p = size(xb, 1);
% general code for n dimensions
if isempty(cfg.slice),  % default slice is first 2 variables, the rest=0
    cfg.slice = [NaN NaN zeros(1, p-2)];
end;
if sum(isnan(cfg.slice)) ~= 2,
    error('Can only expand precisely 2 variables (#NaNs == 2)');
end;
% make grid
expandvars = find(isnan(cfg.slice));    % expand these vars
gr.vertices = cell(p, 1);
gr.size = [];
for i = 1:p,
    if any(i == expandvars),    % expanded var
        gr.vertices{i} = -xb(i):2*xb(i)/cfg.npoints:xb(i);
        gr.size(end+1) = length(gr.vertices{i});
    else                        % collapsed var
        gr.vertices{i} = cfg.slice(i);
    end;
end;
gr.points = flat(gr.vertices);
gr.npoints = size(gr.points, 2);
% compute V function, plot slice
v = zeros(gr.npoints, 1);
for i = 1:gr.npoints, v(i) = fun(gr.points(:, i)); end;
mesh(gr.vertices{expandvars(1)}, gr.vertices{expandvars(2)}, ...
    reshape(v, gr.size)');
