function [u, ord] = pdstatic(x, t, PD, umax, ref)
% PD control in the form of a static feedback; 
% requires that the state signal is (y, ydot) and that y_ref is a constant
% t argument is ignored

% process 'init' mode
if ischar(x) && strcmp(x, 'init'),
    u = [];     % the initial state of the controller
    ord = 0;    % the order of the controller
    return;
end;
% default reference value
if nargin < 6,  ref = 0; end;

% K_P * err (= ref - y) + K_D errdot (= - ydot)
u = PD(1) * (ref - x(1)) - PD(2) * x(2);                % proportional term
% saturate
u = sat(u, -umax, umax);
