%% OO experiments for double int
function malinear_ooconsensus(mode, expids, custombcfg, customacfg)

%% General config
% clear all; close all;

% model and representative initial states
mcfg = struct; 
mcfg.matype = 'fixedconn'; mcfg.rewtype = 'none';
mcfg.type = 'doubleint'; mcfg.dim = 1; mcfg.Ts = 0.5;
mcfg.nagents = 5;
% 1----2-----3-----4-----5
%  \_________/\_________/ 
mcfg.G = [0 1 1 0 0;
          1 0 1 0 0;
          1 1 0 1 1;
          0 0 1 0 1;
          0 0 1 1 0];
x0 = [0 1 2 3 4; -3 -1 0 1 3];

% creaty dummy model to find:
% - "ideal" upper bound Mu
% - Lipschitz constant alphabar of the cost function
m = malinear_problem('model', mcfg);
Atilde = m.A^2; Btilde = [m.A*m.B m.B];
Mu = norm(pinv(Btilde)) * (1+norm(Atilde, 2)) * norm(x0(:), inf);
% override the value with some sufficiently large value
Mu = 100;
alphabar = norm(Btilde);

% batch experiment
BCFG = struct;
BCFG.alg = 'ooconsensus';
BCFG.verb = 5;
% BCFG.in = 'planparamsn';
BCFG.idfields = {'alg', 'optparamsfun', 'problem', 'alpha', 'optparamsmethod', 'optparamsn'};
BCFG.startat = 1; BCFG.repeat = 1;
BCFG.skipcache = 1;         % we don't need to cache the results, they are easy to compute

ACFG = struct;
ACFG.problem = 'malinear_problem';
ACFG.model_params = {mcfg};
ACFG.K = 2;
ACFG.optparams.maxu = repmat(Mu, ACFG.K, 1);
ACFG.optparams.diam = @(urange) alphabar * norm((urange(2, :) - urange(1, :))./2, 2); % for DOO
ACFG.optparams.dmax = @(tau) sqrt(tau); % for SOO
ACFG.trials = 1;
ACFG.maxsteps = 80;
ACFG.steps = [];
% visualization
ACFG.verb = 3;
ACFG.datadir = 'd:\work\m\approxrl\masystems\malinear\malinear_ooconsensus';
ACFG.reset = x0(:);

cd(ACFG.datadir); 


%% Local execution function 
% default custom configs (empty)
if nargin < 3, custombcfg = struct; end;
if nargin < 4, customacfg = struct; end;

    % nested utility function for executing experiment
    function execbatchexp(expid)
        if any(strcmp(expid, expids)),  % if experiment is requested
            bcfg = copyfields(custombcfg, bcfg); 
            acfg = copyfields(customacfg, acfg);
            % check if we need to call 'process' or 'eval'
            if any(strcmp(mode, 'run')), 
                processbatch(bcfg, acfg); 
            elseif any(strcmp(mode, {'disagr', 'exectime'})), 
                bcfg.out = mode;
                evalbatch(bcfg, acfg);
            end;
        end;
    end

%% DOO, influence of n // Dell E6430

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'doobudget';
bcfg.in = 'optparamsn';
bcfg.legendprefix = 'DOO';
acfg = ACFG;
acfg.alpha = .2;
acfg.optparams.method = 'doo';
acfg.optparams.n = [15 25 50 75 100 200 300 400 500 600 700];

execbatchexp(bcfg.idinfix);

%% DOO overestimated Lipschitz constant, influence of n // Dell E6430

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'doolargel';
bcfg.in = 'optparamsn';
bcfg.legendprefix = 'DOO large metric';
acfg = ACFG;
acfg.alpha = .2;
acfg.optparams.method = 'doo';
acfg.optparams.diam = @(urange) 4*alphabar * norm((urange(2, :) - urange(1, :))./2, 2);
acfg.optparams.n = [15 25 50 75 100 200 300 400 500 600 700];

execbatchexp(bcfg.idinfix);

%% SOO, influence of n // Dell E6430

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'soobudget';
bcfg.in = 'optparamsn';
bcfg.legendprefix = 'SOO';
acfg = ACFG;
acfg.alpha = .2;
acfg.optparams.method = 'soo';
acfg.optparams.n = [15 25 50 75 100 200 300 400 500 600 700];

execbatchexp(bcfg.idinfix);


%% DOO, influence of alpha // Dell E6430

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'dooalpha';
bcfg.in = 'alpha';
bcfg.legendprefix = 'DOO';
acfg = ACFG;
acfg.alpha = [.05 .1 .15 .2 .24];
acfg.optparams.method = 'doo';
acfg.optparams.n = 200;
acfg.steps = 61:80;

execbatchexp(bcfg.idinfix);

%% SOO, influence of alpha // Dell E6430

bcfg = BCFG;
bcfg.mode = 'run';
bcfg.idinfix = 'sooalpha';
bcfg.in = 'alpha';
bcfg.legendprefix = 'SOO';
acfg = ACFG;
acfg.alpha = [.05 .1 .15 .2 .24];
acfg.optparams.method = 'soo';
acfg.optparams.n = 500;
acfg.steps = 61:80;

execbatchexp(bcfg.idinfix);

end % main function