%% 1. Circle tracking for single-agent case, as a preparatory step to ensure the leader will work
mcfg = struct; cfg = struct;
mcfg.orientation = 'horiz';
mcfg.rewtype = 'refmodel';
mcfg.refmodel_params = {[.35; .35], .2, 2*pi/3};

cfg.problem = 'rarm_problem';
cfg.model_params = {mcfg}; 
cfg.gamma = 0.98;
cfg.planalg = 'opd';
cfg.planparams.n = 400;
cfg.planparams.U = flat({[-1.5 0 1.5], [-1 0 1]});
cfg.planparams.verb = 1;
cfg.trials = 1;
cfg.maxsteps = 200;
cfg.reset = [0, 0, 0, 0]';
%
cfg.saveplandata = 0;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.datafile = 'rarm_opd_circletracking';
cfg.run = 1;
genmpc(cfg);

%% 2. Simple consensus on all states
macfg = struct; 
macfg.nagents = 3; macfg.G = [0 1 1; 1 0 0; 1 0 0]; % undirected graph 2 -- 1 -- 3
macfg.orientation = 'horizontal';
macfg.Q = diag([1 .1 1 .1]);

cfg = struct;
cfg.problem = 'mararm_problem';
cfg.model_params = {macfg}; 
cfg.gamma = 0.98;
cfg.comm = 'seq';
cfg.planalg = 'maopd_predictneighbors';
cfg.planparams.n = 400;
cfg.planparams.U = repmat({{[-1.5, 0, 1.5], [-1, 0, 1]}}, 1, macfg.nagents);
cfg.planparams.xpredtype = 'comm';
cfg.planparams.verb = 1;
cfg.trials = 1;
cfg.maxsteps = 200;
% regular initial states
% cfg.reset = [0, 0, 0, 0; pi/2, 0, 0, 0; -pi/2, 0, 0, 0]'; cfg.datafile = 'mararm_maopd_cons_G2-1-3_regularangles';
% random initial angles, arms at rest
cfg.reset = 'randangles'; cfg.datafile = 'mararm_maopd_cons_G2-1-3_randangles'; 
%
cfg.saveplandata = 0;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.run = 1;
magenmpc(cfg);

%% 3. Leader-based consensus
% MAS config
macfg = struct; 
% directed graph 1 --> 2 <--> 3
macfg.nagents = 3; macfg.G = [0 0 0; 1 0 1; 0 1 0]; 
macfg.orientation = 'horizontal';
macfg.Q = diag([1 .1 1 .1]);

% leader, single-agent config
lcfg = struct;
lcfg.orientation = 'horiz';
lcfg.rewtype = 'refmodel';
lcfg.refmodel_params = {[.35; .35], .2, 2*pi/3};

cfg = struct;
cfg.problem = 'mararm_problem';
cfg.model_params = {macfg}; 
cfg.gamma = 0.98;
cfg.comm = 'seq';
% leader
cfg.leadproblem = 'rarm_problem';
cfg.leadmodel_params = {lcfg};
cfg.leadplanalg = 'opd';
cfg.leadplanparams.n = 400;
cfg.leadplanparams.U = flat({[-1.5 0 1.5], [-1 0 1]});
% other agents
cfg.planalg = 'maopd_predictneighbors';
cfg.planparams.n = 400;
cfg.planparams.U = repmat({{[-1.5, 0, 1.5], [-1, 0, 1]}}, 1, macfg.nagents);
cfg.planparams.xpredtype = 'comm';
cfg.planparams.verb = 1;
cfg.trials = 1;
cfg.maxsteps = 200;
% notice here the leader starts at 0, above the center agent with index 2 started at 0
cfg.leadreset = [0 0 0 0]';
cfg.reset = [0 0 0 0; pi/2 0 0 0; -pi/2 0 0 0]';
%
cfg.saveplandata = 0;
cfg.verb = 3;
cfg.visualize = 0;
cfg.datadir = pwd;
cfg.datafile = 'mararm_maopd_leadconsensus_G1-)2(-)3';
cfg.run = 1;
magenmpc(cfg);