function varargout = cerbfps_old(cfg)
% Cross-entropy policy search: now OBSOLETE (kept for compatibility); use CERBFPS instead.
%   VARARGOUT = CERBFQI(CFG)
% Inputs:
%   CFG             - structure with fields as commented in the code below
%           can also be given as a string, see he str2cfg
% Outputs:
%   TODO
%
% This is the "old" version of the function, used in the: 
%   JMLR CEPS drafts; ADPRL paper; thesis chapter
% Kept for compatibility reasons.
% This old version contains:
%   - the older (bad) initialization that initializez mean for RBF widths to small values
%   - support for pre-generation of noise sequences
% 
% These have been eliminated or changed in the later version CERBFPS

if nargin < 1, cfg = struct(); end;

% -----------------------------------------------
% Process configuration structure

% default config
% function config
CFG.run = 0;                        % run learning
CFG.resume = 0;                     % resume learning
CFG.replay = 0;                     % replay learned policy
CFG.stats = 0;                      % plot statistics
CFG.evol = 0;                       % detailed solution evolution plots (only works with 2-D problem)
CFG.problem = [];                   % what problem to solve
CFG.datafile = 'cepsdata';          % save data to file
% main algorithm config
CFG.gamma = 0.95;                   % discount factor
CFG.actsel = 'max';                 % action selection method: 
    % 'max' for selecting the action of the RBF with the largest activation
    % 'voting' for summing up activations of RBFs with the same assigned discrete action, and
    % then picking the action with the highest vote
CFG.N = 10;                         % number of basis functions to use
CFG.X0 = NaN;                       % (required) distribution of interesting initial states
CFG.P0 = NaN;                       % weights of score in interesting initial states
CFG.U = NaN;                        % (required) flat action space, pxM
    % (X0 and U can also be supplied by the problem but if not user has to supply them)
% CE config
CFG.ce_N = -1;                      % number of samples to draw at each iteration; -1 for auto, N = min(c * Nphi, Nmax)
CFG.ce_maxN = Inf;                  % maximum value for N (cap)
CFG.ce_c = 5;                       % N = min(c * Nphi, Nmax), when auto, where Nphi number of distr params
CFG.ce_rho = 0.05;                  % best samples at each iteration to use in updates
   % when < 1, the (100*(1-rho))th percentile of best samples will be used
   % when > 1, used as an exact elite number of samples
CFG.ce_alpha = 1;                   % smoothed update coefficient for mean of params distribution
CFG.ce_beta = -1;                   % similar, for variance, -1 = auto takes value of alpha
CFG.ce_gamma = -1;                  % similar, for Bernoulli parameter, -1 = auto takes value of alpha
CFG.ce_maxiter = 50;                % max # iterations for CE optimization
CFG.ce_d = 5;                       % how many iterations should the score be roughly constant for convergence
CFG.ce_eps = .005;                  % threshold for score increase
% CFG.ce_samples = 30;                % how many samples to generate at once to avoid rejects
% Monte-Carlo simulations config for score evaluation
CFG.mc_maxsteps = .005;             % admissible error OR max steps in trajectory to compute returns
    % (when < 1, it's an error level from which the number of steps will be computed)
CFG.mc_noise = 'online';            % 'online' to generate online, 'fixed' to generate fixed noise sequences
CFG.mc_nsim = 10;                   % how many simulations for stochastic systems
% replay config
CFG.interp = 0;                     % interpolate action values using basis function values
CFG.x0 = [];                        % initial state for replay (otherwise the problem default or zeros)
CFG.tend = 30;                      % end time for replay
% stats & figure output config
CFG.postconv = 0;
CFG.plottarget = 'screen';          % 'screen', '', 'latex', or 'beamer'. If 'screen' figures will remain open
CFG.savetheta = 0;                  % save param history in stats
CFG.savedir = '';
CFG.savefig = '';
CFG.ylim = [];              % limits for axes
CFG.xlim = [];
CFG.ytick = [];             % explicit tick spacing
CFG.minmax = 1;             % use min-max plots in stats instead of stdev (default since July 2008)
% display config
CFG.verb = 3;                       % verbosity: the higher, the more detailed the messages displayed
CFG.noplot = 0;                     % whether to suppress figure plots
CFG.silent = 0;                     % suppress all output
CFG.iterdisp = 1;                   % feedback after every iterdisp iterations
CFG.itersave = 1;                   % save after each itersave iterations
CFG.sampledisp = 50;                % feedback after sampledisp samples evaluated in each iteration

CFG.miscinfo = '';                  % field for miscellaneous information
% Early defaults (initialized before calling problem defaults)
ECFG.ce_params = {};                % extra parameters for problem calling in 'ce' mode
ECFG.model_params = {};             % extra parameters for problem calling in 'model' mode

% List of fields that define the problem and therefore may NOT be overwritten on load
KEEPFIELDS = {'gamma', 'N', 'X0', 'U', 'ce_N', 'ce_maxN', 'ce_c', 'ce_rho', ...
    'ce_alpha', 'ce_beta', 'ce_gamma', 'mc_maxsteps', 'actsel'};

% If caller provided string, parse it into a structure
if ischar(cfg),
    cfg = str2cfg(cfg, [fieldnames(CFG); fieldnames(ECFG)]);
end;
% Install 'early' defaults: those that the problem might depend on
cfg = checkparams(cfg, ECFG);
% Try installing problem defaults whenever caller does not specify values
% Problem must be specified for this to work
try     
    if ~isempty(cfg.problem), cfg = checkparams(cfg, feval(cfg.problem, 'ce', cfg.ce_params{:})); end;
catch
%     disp('Error in problem(''ce'')');
%     lasterr;
end;
% Install function defaults for everything else
cfg = checkparams(cfg, CFG);
% Process configuration dependencies
if cfg.silent, cfg.verb = -Inf; cfg.noplot = 1; end;
cfg.grayscale = grayscalefromconfig(cfg);

cfg.noinit = (cfg.resume || cfg.replay || cfg.stats || cfg.evol) && exist([cfg.datafile '.mat'], 'file');

% -----------------------------------------------
% Data loading
if cfg.noinit,        % load data file, making sure that cfg and KEEPFIELDS is not overwritten
    cfg1 = cfg; kf = KEEPFIELDS;
    load(cfg.datafile);
    % Overwrite problem-defining fields from loaded config, keep the rest as in the (current) cfg1;
    % the result becomes the new config
    cfg = copyfields(cfg, cfg1, kf);
    KEEPFIELDS = kf; clear cfg1 kf;
    dispx(['Current data loaded from [' cfg.datafile '].'], cfg.verb, 1);
end;

if (cfg.replay || cfg.stats || cfg.evol) && cfg.postconv,
    % a-posteriori application of convergence test
    K = find(~isnan(Jhat), 1, 'last');
    conv = 0; k = cfg.ce_d;
    while ~conv &&  k < K,
        k = k + 1;
        deltaj = diff( Jhat(max(1, k - cfg.ce_d):k) );
        conv = all(0 <= deltaj) && all(deltaj <= cfg.ce_eps);
    end;
    if conv,
        dispx(['A-posteriori convergence detected at iteration [' num2str(k) '].'], cfg.verb, 1);
        % truncate data arrays after convergence point
        Jhat(k+1:end) = NaN;
        J = J(:, 1:k);
        PHI = {PHI{1:k}};
        G = {G{1:k+1}};
        % recompute optimal with restricted range
        Jstar = max(max(J));
        [istar, kstar] = find(J == Jstar, 1);
        phistar = squeeze(PHI{kstar}(istar, :, :));
    else
        dispx('A-posteriori convergence: not detected.', cfg.verb, 1);
    end;
        
end;

% -----------------------------------------------
% Setup model
if ~cfg.noinit,
    model = feval(cfg.problem, 'model', cfg.model_params{:});    
end;

% Config settings for auto and derived parameters (require model so placed after model init)
% Number of structural parameters: variable-wise center and radius for each basis function
cfg.M = size(cfg.U, 2);
cfg.nbits = ceil(log2(cfg.M));

% (p*N center coordinates + p*N radii) * 2 (2 because each has mean and sigma for Gaussian)
% + N * nbits Bernoulli parameters for the binary action assignments
cfg.Nphi = cfg.N * model.p * 2 * 2 + cfg.N * cfg.nbits; 
if cfg.ce_N < 0,                % auto sample set size
    cfg.ce_N = min(cfg.ce_c * cfg.Nphi, cfg.ce_maxN);
end;
if cfg.mc_maxsteps < 1,         % field specifies error in return, use it to compute maxsteps
    cfg.mc_maxsteps = ceil(log(cfg.mc_maxsteps * (1-cfg.gamma) / model.maxr) / log(cfg.gamma));
end;
if cfg.ce_beta < 0, cfg.ce_beta = cfg.ce_alpha; end;
if cfg.ce_gamma < 0, cfg.ce_gamma = cfg.ce_beta; end;

% get environment (Matlab, hardware) info
cfg.envinfo = getenvx;

% Echo config
dispx('Cross-Entropy policy search called with the following configuration:', cfg.verb, 1);
dispx(cfg, cfg.verb, 1);

% Check file presence
if cfg.run && exist([cfg.datafile '.mat'], 'file'),
    reply = input(['File [' cfg.datafile '] already exists (possible overwrite). Continue? Y/N [N]: '], 's');
    if isempty(reply) || reply == 'N', return; end;
end;       

N = cfg.N; M = cfg.M; nbits = cfg.nbits;	% for easy access
switch cfg.actsel,                          % choose MC evaluation function
    case 'max',
        if isfield(model, 'mc_rbfnearestpolicy_fun'),   % use optimized
            mcfun = model.mc_rbfnearestpolicy_fun;
        	dispx(['Using optimized Monte Carlo score function ' func2str(mcfun)], cfg.verb, 2);
        else        % use default
            mcfun = @mc_rbfnearestpolicy;
        end;
    case 'voting',
        if isfield(model, 'mc_rbfvotingpolicy_fun'),    % use optimized
            mcfun = model.mc_rbfvotingpolicy_fun;
        	dispx(['Using optimized Monte Carlo score function ' func2str(mcfun)], cfg.verb, 2);
        else        % use default
            mcfun = @mc_rbfvotingpolicy;
        end;
end;

% ------------------------------------------------
% process X and U domains, possibly state filter
cfg.xfiltering = isfield(model, 'xfilter');
% compatibility mode for symmetric state and action spaces
if ~isfield(model, 'minx'), model.minx = -model.maxx; end;
if ~isfield(model, 'minu'), model.minu = -model.maxu; end;
if cfg.xfiltering,
    dom.minx = model.minxf; dom.maxx = model.maxxf;
else
    dom.minx = model.minx; dom.maxx = model.maxx;
end;
% compute span and midpoint of (filtered, if such is the case) domain
dom.spanx = dom.maxx - dom.minx; dom.midx = dom.minx + dom.spanx/2;

% -----------------------------------------------
% CE optimization policy search

if cfg.run || cfg.resume,
       
    % -----------------------------------------------
    % Do Q-iteration
    dispx('Performing C-E policy search...', cfg.verb, 0);

    % -----------------------------------------------
    % Initialize if not resuming
    if ~cfg.resume,
        
        % === Deterministic init of centers
        % with this RBFs are concentrated in the middle rather than uniformly distributed, 
        % but it's unclear how to deterministically place RBFs centers uniformly,
        % and I want to provide the same initial conditions to the algorithms for a more
        % meaningful comparison
        g.cmean     = repmat(dom.midx, 1, N);
%         % ensure most of the RBFs stay within the input space
%         g.cstd      = repmat(model.maxx/2, 1, N);
        % ensure good coverage of the input space
        g.cstd      = repmat(dom.spanx/2, 1, N);
        % RBFs width init
        g.radmean   = repmat(dom.spanx/2/(2*(N^(1/model.p)-1)), 1, N);
        % allow width to vary between 0 and double
        g.radstd    = g.radmean/2;
        % Bernoulli parameters - equal probability on every bit
        g.bernp     = 0.5 + zeros(nbits, N);
        G{1} = g;
        
        % parameters, score storage
        PHI = {};
        J = NaN + zeros(cfg.ce_N, cfg.ce_maxiter); Jhat = NaN + zeros(1, cfg.ce_maxiter);
        % placeholder for phi, j at current iteration
        phi = zeros(cfg.ce_N, 2*model.p + nbits, N);    % centers, radii, and action bits stacked on top of each other
        j = zeros(cfg.ce_N, 1);
        
        % initialize noise sequences if needed
        switch cfg.mc_noise,
            case 'online',  % set W to empty to signal no noise sequences
                W = [];
            case 'fixed',   % pre-generated fixed noise sequences
                W = feval(model.genwfun, model, cfg.X0, cfg.mc_nsim, cfg.mc_maxsteps);
            otherwise,
                warning('APPROXL:paramError', ['Noise type ' cfg.mc_noise ' is unknown']);
        end;
        
        k = 1;
        trun = 0;
                
    end;        % initialization IF

    % -----------------------------------------------
    % Main loop of C-E optimization
    
    tmark = cputime;
    conv = 0;
    % preinit c, rad, ubits
    c = zeros(model.p, N); rad = zeros(model.p, N);
	ubits = zeros(nbits, N);
    % determine whether action is power of two for efficient sampling (no invalid samples can
    % exist if the bits encode exactly the number of actions)
    % preinit n=1 matrix for Bernoulli function accordingly
    Mpower2 = (nbits == log2(M));
    if Mpower2,     binon = ones(nbits, N);         % all RBF assignments at once
    else            binon = ones(nbits, 1);         % iteratively draw assignments w/ accept-reject
    end;
    while ~conv && k <= cfg.ce_maxiter,
        
        % draw ce_N samples from the current distribution G{k},
        crejects = 0; radrejects = 0; bitrejects = 0;
        for i = 1 : cfg.ce_N,
            % if #actions is a power of 2, draw directly an assignment of actions
            % otherwise, will do sample rejection below
            if Mpower2,
                ubits = binornd(binon, g.bernp);
            end;
            % Since we are doing accept/reject, it's better to sample per basis function, 
            % as the number of rejects will be smaller like this 
            for ii = 1:N,
                % draw #ii RBF center, rejecting any samples that exceed the 
                % (possibly filtered) state space boundary
                iic = g.cmean(:, ii) + g.cstd(:, ii) .* randn(model.p, 1);
                while any(iic < dom.minx) || any(iic > dom.maxx), 
                    crejects = crejects + 1;
                    iic = g.cmean(:, ii) + g.cstd(:, ii) .* randn(model.p, 1);
                end;
                c(:, ii) = iic;
                % draw #ii RBF radius, rejecting samples with negative or zero components
                iirad = g.radmean(:, ii) + g.radstd(:, ii) .* randn(model.p, 1);
                while any(iirad <= 0), 
                    radrejects = radrejects + 1;
                    iirad = g.radmean(:, ii) + g.radstd(:, ii) .* randn(model.p, 1);
                end;
                rad(:, ii) = iirad;
                if ~Mpower2,
                    % draw action assignments to RBFs, ensuring the 1-based indices do not exceed
                    % the allowed range of discrete actions
                    iiubits = binornd(binon, g.bernp(:, ii));
                    % ensure corresponding 1-based indices do not exceed the #discrete actions
                    while binvec2decx(iiubits, 1) > cfg.M,     
                        bitrejects = bitrejects + 1;
                        iiubits = binornd(binon, g.bernp(:, ii));
                    end;
                    ubits(:, ii) = iiubits;
                end;
            end;
                
            % evaluate the current RBF shapes and action assignments
            if isempty(W),
                j(i) = mcfun(c, rad, binvec2decx(ubits, 1), model, cfg.X0, cfg);
            else            % use pre-generated noise sequences
                j(i) = mcfun(c, rad, binvec2decx(ubits, 1), model, cfg.X0, cfg, W);
            end;
            
            phi(i, :, :) = [c; rad; ubits];
            % visual feedback of algorithm progress
            if ~mod(i, cfg.sampledisp),
                dispx([num2str(i) ' out of ' num2str(cfg.ce_N) ' samples evaluated'], cfg.verb, 3);
            end;
        end;
        
        % sort scores and parameter sets
        [j ind] = sort(j, 1, 'descend');
        phi = phi(ind, :, :);
        % compute last elite sample
        if cfg.ce_rho < 1,      % use percentile
            Jhat(k) = prctile(j, 100*(1-cfg.ce_rho));
            Ne = find(j >= Jhat(k), 1, 'last');
        else                    % use fixed number of elite samples
            Ne = cfg.ce_rho;
            Jhat(k) = j(Ne);	% worst performance in elite samples
        end;
        % compute update of distribution as sampled mean and variance of elite samples
        % (general form of smoothed update)
        % note standard deviations are computed by dividing by number of samples (second input to std flag=1)
        % as opposed to (number of samples - 1)
        g.cmean = (1-cfg.ce_alpha)   .* g.cmean      + cfg.ce_alpha  .* squeeze(mean(phi(1:Ne, 1:model.p, :)));
        g.cstd = (1-cfg.ce_beta)     .* g.cstd       + cfg.ce_beta   .* squeeze(std(phi(1:Ne, 1:model.p, :), 1));
        g.radmean = (1-cfg.ce_alpha) .* g.radmean    + cfg.ce_alpha  .* squeeze(mean(phi(1:Ne, model.p+1:2*model.p, :)));
        g.radstd = (1-cfg.ce_beta)   .* g.radstd     + cfg.ce_beta   .* squeeze(std(phi(1:Ne, model.p+1:2*model.p, :), 1));
        g.bernp = (1-cfg.ce_gamma)   .* g.bernp      + cfg.ce_gamma  .* reshape(mean(phi(1:Ne, 2*model.p+1:end, :)), [], N);

        G{k+1} = g;
        
        % save data from current iteration
        J(:, k) = j; 
        PHI{k} = phi;
        % update stats
        trun = trun + (cputime - tmark);
        
        % check convergence condition
        deltaj = diff( Jhat(max(1, k - cfg.ce_d):k) );
        conv = k > cfg.ce_d && all(0 <= deltaj) && all(deltaj <= cfg.ce_eps);
        
        % visual feedback of algorithm progress
        if ~mod(k, cfg.iterdisp),
            dispx(['k=' num2str(k) ' CE iter done, d(Jhat(k-d:k))=[' num2str(deltaj) ...
                '], J*=' num2str(max(max(J))) ', Jhat(k)=' num2str(Jhat(k)) ...
                ', Jmean(k)=' num2str(mean(J(:, k))) '+-' num2str(std(J(:, k), 1))...
                ', #rej=' num2str(crejects) '+' num2str(radrejects) '+' num2str(bitrejects)], cfg.verb, 2);
        end;
        % data backup
        if ~mod(k, cfg.itersave),            
            save(cfg.datafile);
            dispx(['Data at iter k=' num2str(k) ' saved to [' cfg.datafile '].'], cfg.verb, 1);
        end;
        
        tmark = cputime;
        k = k + 1;
    end;        % WHILE not converged and allowed more iterations

    if conv,	dispx('Convergence detected. Algorithm stopped.', cfg.verb, 0);
    else        dispx(['maxiter=' num2str(cfg.ce_maxiter) ' exhausted. Algorithm stopped'], cfg.verb, 0);
    end;
    
    % return best structural and linear parameter vectors encountered so far
    Jstar = max(max(J));
    [istar, kstar] = find(J == Jstar, 1);
    phistar = squeeze(PHI{kstar}(istar, :, :));
end;


% -----------------------------------------------
% Replay
if cfg.replay,
    
    % ensure deterministic flag is set in the model
    if ~isfield(model, 'det'), model.det = 1; end;
    % ensure phistar is defined -- it's possible this is an intermediate file
    if ~exist('phistar', 'var'),
        Jstar = max(max(J));
        [istar, kstar] = find(J == Jstar, 1);
        phistar = squeeze(PHI{kstar}(istar, :, :));
    end;

	% pick up structure params and action assignments
    cstar = phistar(1:model.p, :);
    radstar = phistar(model.p+1:2*model.p, :);
    uind = binvec2decx(phistar(2*model.p+1:end, :), 1);

    switch cfg.actsel,
        case 'voting',
            [junique, selectors] = ind2selectors(uind);
            % Make phi a column vector such that the sum across colums works properly even when all the RBFs
            % have the same assigned discrete action
            phi = zeros(N+1, 1); % pad with a zero "dummy" for the sums
        case 'max',
            ustar = cfg.U(:, uind);
    end;

    % initial state
    if ~isempty(cfg.x0), 
        if ischar(cfg.x0) && ~isempty(cfg.problem),
            % use the model function to get a string-named initial state
            x0 = feval(cfg.problem, 'x0', cfg.x0);
        else
            x0 = cfg.x0(:);         % specified initial state
        end;
    else
        % try getting default initial state
        try
            x0 = feval(cfg.problem, 'x0');
        catch
            x0 = zeros(model.p, 1);    % zeros
        end;
    end;         
    dispx(['Controlling from x0=' num2str(reshape(x0, 1, [])) ], cfg.verb, 0);

    % init history
    t = 0 : model.Ts : cfg.tend;
    Ns = length(t)-1;       % number of samples / time instances at which control is applied
    x = zeros(model.p, length(t)); x(:, 1) = x0;
    u = zeros(model.q, length(t)); u(:, end) = NaN;
    r = zeros(1, length(t)); r(1) = NaN;

    % steps loop
    for k = 1:Ns,
        % compute optimal action
        switch cfg.actsel,
            case 'voting',      % TODO implement interpolated voting action selection, if needed
                if cfg.xfiltering, phi(1:N) = rbf(model.xfilter(x(:, k), model), N, c, rad);   
                else               phi(1:N) = rbf(x(:, k), N, c, rad);   
                end;
                [actmax imax] = max(sum(phi(selectors), 1));
                u(:, k) = cfg.U(:, junique(imax));
            case 'max',
                if cfg.xfiltering, act = nrbf(model.xfilter(x(:, k), model), cfg.N, cstar, radstar);
                else               act = nrbf(x(:, k), cfg.N, cstar, radstar);
                end;
                if ~cfg.interp,    % pick discrete action corresponding to RBF with largest value
                    [actmax imax] = max(act); clear actmax;
                    u(:, k) = ustar(:, imax);
                else                % interpolate using RBF activations
                    u(:, k) = ustar * act(:);
                end;
        end;
        % TMP FOR HIV
%         if t(k) > 900, u(:, k) = 0; end;
        % apply to system
        [x(:, k+1) r(k+1) terminal] = feval(model.fun, model, x(:, k), u(:, k));
        if terminal, Ns = k; break; end;      % entered terminal state
    end;
 
    % plot history & optionally save figures
    hist.t = t(1:Ns+1); hist.x = x(:, 1:Ns+1); hist.u = u(:, 1:Ns+1); hist.r = r(1:Ns+1);
    if ~cfg.noplot,
        toscreen = strcmp(cfg.plottarget, 'screen') || isempty(cfg.plottarget);
        figh = plothistory(hist);
        if isfield(model, 'plotfun'),
            mpfigh = feval(model.plotfun, hist);
            if ~isempty(mpfigh), figh(end+1) = mpfigh; end;
        end;
        % save if requested
        saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);
        if ~toscreen, close(figh(1)); end;  % don't close the custom plot
    end;

end;        % IF replay

% -----------------------------------------------
% Plot code

figh = [];

% plot settings, labels etc.
if cfg.stats || cfg.evol,
    % grayscale styles
    gs.mean = {'k'};
    gs.max = {'k', 'LineWidth', 2};
    gs.min = {'k:', 'LineWidth', 1};
    gs.quantile = { 'k--', 'LineWidth', 2};
    gs.colors = {'r', 'g', 'b', 'c', 'm', 'y', 'k', [.5 .5 .5], [0 .1 .7], [.1 .7 0], [.7 0 .1]};
    % black to dirty white
    gs.cm = gray(96); gs.cm = gs.cm(24:end);
    cs = gs;
    cs.cm = jet;
    % set style
    if cfg.grayscale, sty = gs; 
    else sty = cs; end;    
    
    % readable labels
    commonprop = {'Interpreter', 'Latex', 'FontSize',13};
    rl.avg = 'Mean score'; 
    rl.quantile = '(1-\rho_{CE}) quantile, \lambda_\tau'; 
    rl.max = 'Max score';
    rl.min = 'Min score';
    rl.x = {'Iteration \tau'};
    rl.y = {'Score'};
    labels = rl;    % no psfrag labels
end;

% -----------------------------------------------
% Synthetic stats
if cfg.stats && ~cfg.noplot, 
    K = find(~isnan(Jhat), 1, 'last');
    toscreen = strcmp(cfg.plottarget, 'screen') || isempty(cfg.plottarget);
    figh(end+1) = figure; hold on; box on;
    setlimits(cfg);
    
    if cfg.minmax,
        plot(1:K, mean(J(:, 1:K)), sty.mean{:});  
        plot(1:K,max(J(:, 1:K)), sty.max{:});
        plot(1:K,min(J(:, 1:K)), sty.min{:});
        plot(1:K, Jhat(1:K), sty.quantile{:});
        lh = legend(labels.avg, labels.max, labels.min, labels.quantile); 
    else
        % NOTE USING STD AS THE ERRORBAR IS MEANINGLESS (need to attach confidence interval to
        % it)
        errorbar(1:K, mean(J(:, 1:K)), std(J(:, 1:K), 1), sty.mean{:});  
        plot(1:K,max(J(:, 1:K)),  sty.max{:});
        plot(1:K, Jhat(1:K), sty.quantile{:});
        lh = legend(labels.avg, labels.max, labels.quantile); 
    end;
    xlabel(labels.x{:}); ylabel(labels.y{:}); 
    set(lh, 'Location', 'Best');
    if toscreen, title(['N=' num2str(cfg.N)]); end;
    set(figh, 'Name', cfg.datafile, 'NumberTitle', 'off');
    
    saveplot(figh, [cfg.savedir cfg.savefig], cfg.plottarget);
end;


% -----------------------------------------------
% Evolution
if cfg.evol && ~cfg.noplot,  
    K = find(~isnan(Jhat), 1, 'last');
    
    figh(end+1) = figure;
    set(figh, 'Name', cfg.datafile, 'NumberTitle', 'off');
    if model.p == 2, subsz = 320;
    else subsz = 210;
    end;

    kstep = 1;
    for k = 1:kstep:K,
        % ---- synthetic stats up to current iter
        subplot(subsz+1); cla; hold on;
        if k >= 2,
            errorbar(1:k, mean(J(:, 1:k)), std(J(:, 1:k), 1), sty.mean{:});  
            plot(1:k,max(J(:, 1:k)),  sty.max{:});
            plot(1:k, Jhat(1:k), sty.quantile{:});
            xlabel(labels.x{:}); ylabel(labels.y{:}); 
        end;
        
        % histogram of sample performances
        subplot(subsz+2); cla;
        [n, xout] = feval('hist', (J(:, k)), 20);
        bar(xout, n);
        set(gca, 'xlim', [min(min(J)) max(max(J))]);
        set(gca, 'ylim', [0 max(n)*1.1]);
        hold on;
        plot([Jhat(k) Jhat(k)], [0 max(n)*1.1], 'Color', 'r', 'LineWidth', 2);
        xlabel('Score');
        ylabel('# samples');
        title('Sample performance histogram');
        
        if model.p == 2,    % more details
            g = G{k};
            % ---- centers distributions
            subplot(subsz+3); cla; hold on;
            for i = 1:N,
                plot(g.cmean(1, i), g.cmean(2, i), 'Marker', 'o', ...
                    'MarkerFaceColor', sty.colors{i}, 'MarkerEdgeColor', sty.colors{i});
                rectangle('Position', [g.cmean(1,i)-g.cstd(1,i), g.cmean(2,i)-g.cstd(2,i), 2*g.cstd(1,i), 2*g.cstd(2,i)], ...
                    'Curvature', [1, 1], ...        % make it an ellipse
                    'EdgeColor', sty.colors{i});
            end;
            p3cfg.xlim = [model.minx(1) model.maxx(1)];
            p3cfg.ylim = [model.minx(2) model.maxx(2)];
            setlimits(p3cfg);
            xlabel(labels.x{:}); ylabel(labels.y{:}); 
            title('Center distributions');

            % ---- radii distributions
            subplot(subsz+4); cla; hold on;
            for i = 1:N,
                plot(g.radmean(1, i), g.radmean(2, i), 'Marker', 'o', ...
                    'MarkerFaceColor', sty.colors{i}, 'MarkerEdgeColor', sty.colors{i});
                rectangle('Position', [g.radmean(1,i)-g.radstd(1,i), g.radmean(2,i)-g.radstd(2,i), 2*g.radstd(1,i), 2*g.radstd(2,i)], ...
                    'Curvature', [1, 1], ...        % make it an ellipse
                    'EdgeColor', sty.colors{i});
            end;
            xlabel(labels.x{:}); ylabel(labels.y{:}); 
            title('Radii distributions');

            % ---- action probabilities
            subplot(subsz+5); cla; hold on;
            up = zeros(N, M);
            for i = 1:N,
                for j = 1:M,
                    ub = dec2binvec(j-1, nbits);
                    p = 1;
                    if any(ub),
                        p = p * prod(g.bernp(ub, i));
                    end;
                    if ~all(ub),
                        p = p * prod(1-g.bernp(~ub, i));
                    end;
                    up(i, j) = p;
                end;
            end;
            bar(1:N, up, 'stack');
            set(gca, 'Ylim', [0, 1.1]);
            xlabel('RBF index'); ylabel('Action probabilities');
            title('Action assignment distributions');

            % ---- policy
            % Plot 6: actual policy (using plot function supplied by model)
            if exist('cleanedup', 'var') && cleanedup > 1,
                disp('Warning! Cannot plot policy, sample history was cleaned up (cleanup level > 1).');
            else
                % obtain parameter vector for policy plot
                if exist('cleanedup', 'var') && cleanedup <= 1, 
                    % the file has been cleaned up with level 1, use the pre-computed elite sample
                    phistar = squeeze(PHIelite(k, :, :));
                else
                    % find elite sample
                    if cfg.ce_rho < 1,      % use percentile
                        j = J(:, k); Ne = find(j >= Jhat(k), 1, 'last');
                    else                    % use fixed number of elite samples
                        Ne = cfg.ce_rho;
                    end;
                    phistar = squeeze(PHI{k}(Ne, :, :));
                end;
                subplot(subsz+6); cla;
                p2cfg.rbfdirecth = 1; p2cfg.basis = 1; p2cfg.datasource = 'caller'; p2cfg.addtocrt = 1;
                p2cfg.posstep = 0.05; p2cfg.basisc = sty.colors;
                feval(model.plotfun, p2cfg);
            end;
        end;

        drawnow;
        if model.p ~= 2,
            pause(.1);
        end;
    end;
    
%     if cfg.ce_rho < 1,
%         for k = 1:K,
%             j = J(:, k);
%             Ne(k) = find(j >= Jhat(k), 1, 'last');
%         end;
%         figh(end+1) = figure; figure;
%         plot(1:K, Ne);
%     end;

end;


% -----------------------------------------------
% Backup data
if cfg.run || cfg.resume, 
    % save into the same directory as the problem
    savedir = fileparts(which(cfg.problem));
    % if default file name is used, generate a unique file name
    if strcmp(cfg.datafile, CFG.datafile),
        c = fix(clock);
        for i = 1:length(c),
            cfg.datafile = [cfg.datafile '-' num2str(c(i), '%02d')];
        end;
        delete([CFG.datafile '.mat']);
    elseif ~strcmp(savedir, pwd)
        delete([cfg.datafile '.mat']);   % need to save in a different directory, so delete anyway
    end;    % otherwise just overwrite
    cfg.datafile = [savedir '\' cfg.datafile];
    save(cfg.datafile);
    dispx(['Cross-entropy policy search finished. Data was saved to [' cfg.datafile '].'], cfg.verb, 1);
end;


% -----------------------------------------------
% set output
if cfg.run || cfg.resume,       % output optimal parameter vectors, optimal score
    varargout = {phistar, Jstar};
elseif cfg.replay               % output history and possibly figure handles
    fig = ~cfg.noplot && (strcmp(cfg.plottarget, 'screen') || isempty(cfg.plottarget));
    if fig,         varargout = {hist, figh};
    else            varargout = {hist, []};
    end;
elseif cfg.stats,
    varargout = {figh};
end;


% END fuzzyqi() RETURNING varargout =================================================================
