function [xplus, rplus, terminal, pplus] = cleanrob_mdp(m, x, u)
%  Implements the dynamics of the cleaning robot problem.

% special case: terminal state -- cannot be changed and always leads to 0 reward
if (x == m.x_energy) || (x == m.x_can), 
    xplus = x;
    rplus = 0;
    terminal = 1;
    if nargout == 4, pplus = 1; end;
    return;
end;

if m.det,
    xplus = sat(x+u, m.minx, m.maxx);
    [rplus, terminal] = outcome(m, xplus);
    if nargout == 4, pplus = 1; end;
else
    % stochastic case
    % compute all outcome states (as intended, still, reverse) and their probabilities
    x1 = [sat(x+u, m.minx, m.maxx) x sat(x-u, m.minx, m.maxx)];
    p1 = [1-sum(m.prob_char) m.prob_char(1) m.prob_char(2)];
    % remove any zero-probability outcomes
    x1 = x1(p1 > 0); p1 = p1(p1 > 0);
    % remove any duplicates
    xplus = unique(x1);
    pplus = 0*xplus; rplus = 0*xplus; terminal = 0*xplus;
    for i = 1:length(xplus), 
        pplus(i) = sum(p1(x1 == xplus(i))); 
        [rplus(i), terminal(i)] = outcome(m, xplus(i));
    end;
    % choose one outcome randomly if called with only 3 output arguments
    if nargout < 4,
        iplus = randp(pplus, 1);
        xplus = xplus(iplus);
        rplus = rplus(iplus);
        terminal = terminal(iplus);
    end;
end;


end % main function


function [rplus, terminal] = outcome(m, xplus)
switch xplus,
    case m.x_can,
        rplus = m.rew_can;
        terminal = 1;
    case m.x_energy,
        rplus = m.rew_energy; 
        terminal = 1;
    otherwise,
        rplus = m.rew_step;
        terminal = 0;
end;    
end     % outcome local function
