function [umin, umax] = dc_linenvelope(x, R, b, db, maxu, du)
% Init mode:
%   ARGS = DC_LINENVELOPE(INIT, MODEL)
% Normal mode:
%   [UMIN, UMAX] = DC_LINENVELOPE(X, ARGS{:})
%   [UMIN, UMAX] = DC_LINENVELOPE(X, R, B, DB, MAXU, DU)

if ischar(x) && strcmp(x, 'init'),
    model = R;
    % rotate to align the constraints with the axes
    alpha = pi/2 - atan(model.maxx(2) / model.maxx(1));  % angle to rotate CW
    R = [cos(-alpha) -sin(-alpha); sin(-alpha) cos(-alpha)];
    % new horiz coord of the line passing through (maxx(1), 0) -- (0, maxx(2))
    b = model.maxx(1) * cos(alpha); 
    b = b * 1;
    db = b / 2;
    du = model.maxu/20;
    umin = {R, b, db, model.maxu, du};
    return;
end;

x = R(1, :) * x;        % rotate so that region bounds are horizontal
if x > b,               % min-action region; upper bound (close to) -maxu
    umin = -maxu; umax = -maxu + du;
elseif x < -b,          % max-action region; lower bound (close to) maxu
    umin = maxu - du; umax = maxu; 
elseif x > b - db,   % increasing min-action region; upper bound linearly increasing to maxu
    umin = -maxu; umax = -maxu + du + (b - x)/db * (2*maxu - du);
elseif x < -b + db,  % increasing max-action region; lower bound linearly increasing to maxu
    umin = -maxu + (-x - b + db)/db * (2*maxu - du); umax = maxu;
else
    umin = -maxu; umax = maxu;
end;
