% What experiments to process and in what mode
exp = '*inv';
mode = 'plot';       % 'run', 'sol', 'stats', 'cleanup'

% main cfg
cfg = struct;
cfg.(mode) = 2;
cfg.gamma = .95;
cfg.problem = 'dc_problem';
cfg.model_params = {'lqr Q=[5,.02] R=.05', 'lin'}; % Larger LQR weights, lin model
cfg.samesamples = 0;        % does not make sense when exploration differs
cfg.datadir = pwd;
cfg.show = {'explor', 'rep'};
% cfg.Ns = 5000;
cfg.Ns = 10000;     % for this value of Ns also ridge regression exp are available
% cfg.Ns = [5000 10000];
cfg.xspace = 'eq'; cfg.uspace = 'log';
cfg.maxiter  = 50;
cfg.repeat = 5;
% perf eval
cfg.perf = 1;                       % turn off for now, we care about iter only
cfg.X0 = 'coarse';                  % when evaluating performance, use coarse grid
cfg.mc_eps = 0.1;      
% replay
cfg.x0 = [pi;0]; cfg.tend = 1.5;
% plot
if strcmp(mode, 'plot'), 
    cfg.stats = 1;
    cfg.plot = {'J', 'iter'};
%     cfg.plottarget = 'latex'; 
    if cfg.Ns == 10000,
        cfg.savefig = 'dcsamph';
        cfg.savedir = 'D:\Work\tex\reports\2008-02-11_3wr\img\';
        cfg.ylim_matrix = [-280 -260; 0 40];
    elseif cfg.Ns == 5000,
        cfg.savefig = 'dcsamphNs5000';
        cfg.savedir = 'D:\Work\tex\reports\2008-02-11_3wr\img\unused\';
    end;
    cfg.xlim = [0 .9];
    cfg.xtick = [0 .1 .3 .5 .7 .9];
    cfg.figsize = [450 350];
end;
if cfg.stats, cfg.verb = 0; else cfg.verb = 5; end;

% Compute discounted LQR controller
model = dc_problem('model', cfg.model_params{:}); 
[K, X] = gammadare(struct('model', model, 'gamma', cfg.gamma));
% Config for using prior, policy
hcfg = cfg;
hcfg.samesamples = 0;   % cannot use the same samples
hcfg.samples_params.policy = @lsf;
hcfg.samples_params.policyargs = {K, model.maxu};
hcfg.explor = [0 .1 .3 .5 .7 .9];
hcfg.sstest = {'N', 'M', 'Ns', 'explor'};   % THIS IS NOT TESTED -- but only active in run mode
expout = 0;

% firstly, regular inversion

% -------- TRIANG experiments, regular inversion
% TRIANGULAR discrete / policy
thexp = hcfg;
thexp.type     = 'triang'; thexp.miscinfo = 'th_inv';
thexp.N        = 9; thexp.M = 5;
thexp.method   = 'randhdisc';
thexp.addtocrt = 0; thexp.expprefix = 'prior,';     % only effective in stats+plot mode
if checkrunexp(thexp.miscinfo, exp), expout = lspiexp(thexp); end;
% TRIANGULAR discrete
texp = cfg;
texp.type     = 'triang'; texp.miscinfo = 't_inv';
texp.N        = 9; texp.M = 5; 
texp.method   = 'randdisc';
texp.addtocrt = expout; texp.expprefix = 'uniform,';     % only effective in stats+plot mode
if checkrunexp(texp.miscinfo, exp), expout = lspiexp(texp); end;
% TRIANGULAR quad / policy
tphexp = hcfg;
tphexp.type     = 'triangpoly'; tphexp.miscinfo = 'tph_inv';
tphexp.N        = 9; tphexp.M = 2;
tphexp.method   = 'randh';
tphexp.addtocrt = 0; tphexp.expprefix = 'prior,';     % only effective in stats+plot mode
if checkrunexp(tphexp.miscinfo, exp), expout = lspiexp(tphexp); end;
% TRIANGULAR quad
tpexp = cfg;
tpexp.type     = 'triangpoly'; tpexp.miscinfo = 'tp_inv';
tpexp.N        = 9; tpexp.M = 2;
tpexp.method   = 'rand';
tpexp.addtocrt = expout; tpexp.expprefix = 'uniform,';     % only effective in stats+plot mode
if checkrunexp(tpexp.miscinfo, exp), expout = lspiexp(tpexp); end;

% -------- RBF experiments, regular inversion
% RBF discrete H
rhexp = hcfg;
rhexp.type     = 'rbfdisc'; rhexp.miscinfo = 'rh_inv';
rhexp.N        = 9; rhexp.M = 5;
rhexp.method   = 'randhdisc';
rhexp.usesamples = thexp;
rhexp.addtocrt = 0; rhexp.expprefix = 'prior,';     % only effective in stats+plot mode
if checkrunexp(rhexp.miscinfo, exp), expout = lspiexp(rhexp); end;
% RBF discrete
rexp = cfg;
rexp.type     = 'rbfdisc'; rexp.miscinfo = 'r_inv';
rexp.N        = 9; rexp.M = 5; 
rexp.method   = 'randdisc';
rexp.usesamples = texp;
rexp.addtocrt = expout; rexp.expprefix = 'uniform,';     % only effective in stats+plot mode
if checkrunexp(rexp.miscinfo, exp), expout = lspiexp(rexp); end;
% RBF poly H
rphexp = hcfg;
rphexp.type     = 'rbfpoly'; rphexp.miscinfo = 'rph_inv';
rphexp.N        = 9; rphexp.M = 2;
rphexp.method   = 'randh';
rphexp.usesamples = tphexp;
rphexp.addtocrt = 0; rphexp.expprefix = 'prior,';     % only effective in stats+plot mode
if checkrunexp(rphexp.miscinfo, exp), expout = lspiexp(rphexp); end;
% RBF poly
rpexp = cfg;
rpexp.type     = 'rbfpoly'; rpexp.miscinfo = 'rp_inv';
rpexp.N        = 9; rpexp.M = 2;
rpexp.method   = 'rand';
rpexp.usesamples = tpexp;
rpexp.addtocrt = expout; rpexp.expprefix = 'uniform,';     % only effective in stats+plot mode
if checkrunexp(rpexp.miscinfo, exp), expout = lspiexp(rpexp); end;

% Then, ridge regression
cfgrr = cfg; cfgrr.Ns = 10000;      % only this # of samples for ridge regression
cfgrr.lspi_params = struct('invdelta', 1e-5); cfgrr.infix = 'rr1m5';
hcfgrr = hcfg; hcfgrr.Ns = 10000;   % only this # of samples for ridge regression
hcfgrr.lspi_params = struct('invdelta', 1e-5); hcfgrr.infix = 'rr1m5';

% -------- TRIANG experiments, ridge regression
% TRIANGULAR discrete / policy
th_rrexp = hcfgrr;
th_rrexp.type     = 'triang'; th_rrexp.miscinfo = 'th_rr';
th_rrexp.N        = 9; th_rrexp.M = 5;
th_rrexp.method   = 'randhdisc';
th_rrexp.usesamples = thexp;
th_rrexp.addtocrt = 0; th_rrexp.expprefix = 'prior,';     % only effective in stats+plot mode
if checkrunexp(th_rrexp.miscinfo, exp), expout = lspiexp(th_rrexp); end;
% TRIANGULAR discrete
t_rrexp = cfgrr;
t_rrexp.type     = 'triang'; t_rrexp.miscinfo = 't_rr';
t_rrexp.N        = 9; t_rrexp.M = 5; 
t_rrexp.method   = 'randdisc';
t_rrexp.usesamples = texp;
t_rrexp.addtocrt = expout; t_rrexp.expprefix = 'uniform,';     % only effective in stats+plot mode
if checkrunexp(t_rrexp.miscinfo, exp), expout = lspiexp(t_rrexp); end;
% TRIANGULAR quad
tp_rrexp = cfgrr;
tp_rrexp.type     = 'triangpoly'; tp_rrexp.miscinfo = 'tp_rr';
tp_rrexp.N        = 9; tp_rrexp.M = 2;
tp_rrexp.method   = 'rand';
tp_rrexp.usesamples = tpexp;
tp_rrexp.addtocrt = 0; tp_rrexp.expprefix = 'prior,';     % only effective in stats+plot mode
if checkrunexp(tp_rrexp.miscinfo, exp), expout = lspiexp(tp_rrexp); end;
% TRIANGULAR quad / policy
tph_rrexp = hcfgrr;
tph_rrexp.type     = 'triangpoly'; tph_rrexp.miscinfo = 'tph_rr';
tph_rrexp.N        = 9; tph_rrexp.M = 2;
tph_rrexp.method   = 'randh';
tph_rrexp.usesamples = tphexp;
tph_rrexp.addtocrt = expout; tph_rrexp.expprefix = 'uniform,';     % only effective in stats+plot mode
if checkrunexp(tph_rrexp.miscinfo, exp), expout = lspiexp(tph_rrexp); end;

% -------- RBF experiments, ridge regression
% RBF discrete H
rh_rrexp = hcfgrr;
rh_rrexp.type     = 'rbfdisc'; rh_rrexp.miscinfo = 'rh_rr';
rh_rrexp.N        = 9; rh_rrexp.M = 5;
rh_rrexp.method   = 'randhdisc';
rh_rrexp.usesamples = thexp;
rh_rrexp.addtocrt = 0; rh_rrexp.expprefix = 'prior,';     % only effective in stats+plot mode
if checkrunexp(rh_rrexp.miscinfo, exp), expout = lspiexp(rh_rrexp); end;
% RBF discrete
r_rrexp = cfgrr;
r_rrexp.type     = 'rbfdisc'; r_rrexp.miscinfo = 'r_rr';
r_rrexp.N        = 9; r_rrexp.M = 5; 
r_rrexp.method   = 'randdisc';
r_rrexp.usesamples = texp;
r_rrexp.addtocrt = expout; r_rrexp.expprefix = 'uniform,';     % only effective in stats+plot mode
if checkrunexp(r_rrexp.miscinfo, exp), expout = lspiexp(r_rrexp); end;
% RBF poly H
rph_rrexp = hcfgrr;
rph_rrexp.type     = 'rbfpoly'; rph_rrexp.miscinfo = 'rph_rr';
rph_rrexp.N        = 9; rph_rrexp.M = 2;
rph_rrexp.method   = 'randh';
rph_rrexp.usesamples = tphexp;
rph_rrexp.addtocrt = 0; rph_rrexp.expprefix = 'prior,';     % only effective in stats+plot mode
if checkrunexp(rph_rrexp.miscinfo, exp), expout = lspiexp(rph_rrexp); end;
% RBF poly
rp_rrexp = cfgrr;
rp_rrexp.type     = 'rbfpoly'; rp_rrexp.miscinfo = 'rp_rr';
rp_rrexp.N        = 9; rp_rrexp.M = 2;
rp_rrexp.method   = 'rand';
rp_rrexp.usesamples = tpexp;
rp_rrexp.addtocrt = expout; rp_rrexp.expprefix = 'uniform,';     % only effective in stats+plot mode
if checkrunexp(rp_rrexp.miscinfo, exp), expout = lspiexp(rp_rrexp); end;

