function varargout = dd_nearoptimal(cfg)
% Computes near-optimal policy for dd problem, and its return on a set of
% initial states

if nargin < 1, cfg = struct(); end;

% CFG.problem = 'ddi_problem';  % hardcoded for the DD problem
CFG.xgrids = [];
% learning params -- if learning has to be performed
CFG.gamma = 0.95;
CFG.eps = 1e-7;
CFG.maxiter = 1000;
CFG.run = 1;
CFG.initstepdisp = .1;
CFG.itersave = 50;
CFG.serial = 0;
CFG.datafile = [];
% default grid # points
CFG.xres = 300;
% on what grid of states to evaluate the return
CFG.X0 = 'p01s02';
CFG.mc_eps = 0.005;
CFG.model_params = {'QUAD2'}; % Possible values: quad, disc, QUAD2, lin

% If caller provided string, parse it into a structure
if ischar(cfg),
    cfg = str2cfg(cfg, fieldnames(CFG));
end;
% Install 'early' defaults: those that the problem might depend on
cfg = checkparams(cfg, CFG);

m = ddi_problem('model');
xrange = m.maxx .* 2;

% dependent defaults
if isempty(cfg.xgrids),
    cfg.xgrids = {-m.maxx(1):xrange(1)/cfg.xres:m.maxx(1), -m.maxx(2):xrange(2)/cfg.xres:m.maxx(2)};
end;
if isempty(cfg.datafile),
    cfg.datafile = ['dd_nearopt_' cfg.model_params{1}];
end;

if ~exist([cfg.datafile '.mat'], 'file'),
    thetastar = fuzzyqi(cfg);
end;
load(cfg.datafile, 'theta', 'model', 'XMFS'); thetastar = theta; clear theta;

cfg.mc_maxsteps = ceil(log(cfg.mc_eps * (1-cfg.gamma) / model.maxr) / log(cfg.gamma));

load(cfg.datafile, 'model', 'XMFS');
cecfg = ddi_problem('ce', cfg.X0);
cfg.U = cecfg.U;
[eJ J] = mc_fuzzyq(XMFS, thetastar, model, cecfg.X0, cfg);

varargout = {eJ, J};