function [sys,x0,str,ts] = sfserial(t,x,u,flag,Ts,com)
%SFSERIAL Serial output to robot from Simulink.
%    Ts  ... sampling period in [s]
%    com ... string denoting the serial port (e.g. 'COM1')

%   (c) Robert Babuska, March 2005

switch flag
  case 0; [sys,x0,str,ts]=mdlInitializeSizes(Ts,com);
  case 3; sys = mdlOutputs(u,x,com);
  case 9; sys = mdlTerminate(com);
  otherwise; sys=[];
end

function [sys,x0,str,ts]=mdlInitializeSizes(Ts,com)
sizes = simsizes;
sizes.NumContStates  = 0;           % no need for states
sizes.NumDiscStates  = 0;           % no need for states
sizes.NumOutputs     = 0;           % no outputs
sizes.NumInputs      = 7;           % 6 actuators
sizes.DirFeedthrough = 1;
sizes.NumSampleTimes = 1;
sys = simsizes(sizes);
x0 = []; str = [];
ts = [-1 0];                        % Set sample time
if length(Ts)>0 ts(1)=Ts(1); end;
if length(Ts)>1 ts(2)=Ts(2); end;
fopen(com);                         % open serial port

function sys = mdlOutputs(u,x,com)
if u(7),
    fwrite(com,40,'uint8');         % laser on
else    
    fwrite(com,56,'uint8');         % laser off
end;    
fwrite(com,0,'uint8');              % control byte for send all actuators
fwrite(com,u(1:6),'uint8');         % write 6 bytes to 6 actuators
sys = [];

function sys = mdlTerminate(com)
fwrite(com,56,'uint8');             % laser off
fwrite(com,0,'uint8');              % control byte for send all actuators
fwrite(com,127*ones(6,1),'uint8');  % reset all 6 actuators to 0 (127 bin)
fclose(com);                        % close serial port
sys = [];
