% %%%% Local function to draw the hillcar for the "schematic" option
function hillcar_draw(cfg)
    CFG.plottarget = 'screen';      % 'latex', 'beamer', 'screen', ''
    CFG.savedir = ''; % 'D:\Work\tex\papers\alamas07\img\';
                            % path for saving figure
    CFG.savefig = 'hc_schematic';   % filename for saving figure
    CFG.car = 1;                    % put car & associated objects (if 0 just draw background)
    CFG.carp = 0.25;                 % car position
    CFG = setfigprop(CFG, 'addfields');
    CFG.figsize = [350 250];
    CFG.hillaspectratio = 0;        % 1 to preserve aspect ratio

    % 2008-08-01 -- removed latex code and replaced with simple labels

    % process config
    if ischar(cfg), cfg = str2cfg(cfg, fieldnames(CFG)); end;
    cfg = checkparams(cfg, CFG);

    cfg.grayscale = grayscalefromconfig(cfg);

    gs.ccolor = 'k'; gs.hcolor = 'k';
    cs.ccolor = 'r'; cs.hcolor = [0 .5 0];

    if cfg.grayscale,
        sty = gs;
    else
        sty = cs;
    end;

    p = -1:.005:1;
    h = 0*p;

    neg = p < 0;
    pos = p >= 0;
    h(neg) = p(neg) + p(neg) .* p (neg);
    h(pos) = p(pos) ./ sqrt(1 + 5 * p(pos));

    xc = p(round(10000*p) == round(10000*cfg.carp));
    hc = h(round(10000*p) == round(10000*cfg.carp));

    common = {'Interpreter', 'LaTeX', 'FontSize', 14};

    fcfg.size = cfg.figsize;
    figh = figurex(fcfg); clf; box on;
    plot(p, h, 'Color', sty.hcolor, 'LineWidth', 2); grid on; hold on;

    % xlabel('$p$', common{:}); ylabel('$H(p)$', common{:});
    xlabel('p'); ylabel('H(p)');

    if cfg.hillaspectratio, set(gca, 'DataAspectRatio', [1 1 1]); end;

    if cfg.car,
        plot(xc, hc, 'Color', sty.ccolor, 'Marker', 'o', 'MarkerSize', 10, 'MarkerFaceColor', sty.ccolor);

        % axis equal
        set(gcf, 'Units', 'normalized');
        [fx fy] = axescoord2figurecoord([xc xc+0.4], [hc hc]);
        annotation('arrow', fx, fy, 'Color', 'k');
        [fx fy] = axescoord2figurecoord([xc xc], [hc hc-0.3]);
        annotation('arrow', fx, fy, 'Color', 'k');

        text(xc+0.45, hc, 'u');
    %     text(xc-.05, hc-0.25, 'mg');
        text(xc+.05, hc-0.3, 'mg');
    %     text(xc+0.5, hc, '$u$', common{:});
    %     text(xc-.1, hc-0.3, '$mg$', common{:});
    end;

    % >= 0 derivative
    % plot(p, (1+5.*p.*p).^-1.5);
    saveplot(figh(end), [cfg.savedir cfg.savefig], cfg.plottarget);

end     % hillcar_draw