function rarm_fittedqi(cfg)
% Fitted QI experiments for the robotic arm.
% Vertical and underactuated (requiring swingup)
% Used in the Automatica FZQI paper. Mainly unsuccessful

% MAT File names contain Ns, and any non-default values for other
% parameters (see below for defaults)
% Particularly interesting experiments:
% 1. rarm_fittedqi_grid_iter100
%   Used in Automatica FuzzyQI paper; uses the same grids as Fuzzy QI
% 2. rarm_fittedqi_gridM3_ntrees25_iter100_{singletree|multitree}
%   Comparison to single-tree ensemble approximator (not one per discrete
%   action)
%   Had to reduce #discrete actions to 3 per axis and #trees to 25 to make
%   this feasible.
%   The "multitree" is the one-tree-per-action baseline, with the same
%   parameters

% FIG files with policy slices are also saved (due to high computational
% cost of regenerating them)
% Also some _replay.mat file contain replay histories

CFG = struct;
% problem config
CFG.mode = 'run';
CFG.problem = 'rarm_problem';
CFG.modelcfg.orientation = 'vert';
CFG.modelcfg.maxtau = [3 1];
CFG.model_params = {CFG.modelcfg};
% algorithm config
CFG.M = 5;              % number of (log-spaced) discrete actions for each action variable
CFG.gamma = .98;        % discount factor
CFG.regmethod = 'extratrees';
CFG.singlereg = 0;
CFG.maxiter = 50;       % max # iterations
CFG.trees_ntrees = 50;    % # of trees for ensemble methods
CFG.trees_k = [];         % k param for extra trees; [] = defaults to #of regression vars
CFG.trees_nmin = 2;       % nmin param for extra trees
% samples config
CFG.method = 'randdisc';
CFG.Ns = 100000;
% prepare grid sample generation in case method is 'grid'
% by default (with M = 5 and the xgrids below), grids are the same 
% as used in the Automatica paper application of fuzzyQiter
CFG.xgrids = [13 7 13 7];
CFG.ugrids = [];
CFG.gridfun = @symloggrid;
% output config
CFG.datadir = [pwd '\'];
CFG.datafile = 'rarm_fittedqi';
CFG.itersave = 10;
CFG.verb = 3;

if nargin < 1, cfg = CFG; 
else cfg = parseconfig(cfg, CFG);
end;

cfg.U = {symloggrid(cfg.M, cfg.modelcfg.maxtau(1)), symloggrid(cfg.M, cfg.modelcfg.maxtau(2))};

if isempty(cfg.ugrids), cfg.ugrids = [cfg.M cfg.M]; end;  % default U grid size

% copy top-level sample config fields to samples substructure
cfg.samples.method = cfg.method;
cfg.samples.N = cfg.Ns;
cfg.samples.xgrids = cfg.xgrids;
cfg.samples.ugrids = cfg.ugrids;
cfg.samples.gridfun = cfg.gridfun;

% execute algorithm
rcfg = cfg;
rcfg.(cfg.mode) = 1;

rcfg.samples
fittedqi(rcfg);
