function rarm_olspi(exp, mode, c)
% LSPI online experiments for robotic arm. This file handles the experiments used 
% in the ICML draft; as well as later experiments (if any).

% What experiments to process and in what mode (defaults)
if nargin < 1, exp = ''; end;
if nargin < 2, mode = 'run'; end;
if nargin < 3, c = struct; end;

% common config, including some fields specific to offline LSPI
CCFG = struct;
CCFG.(mode) = 1;
CCFG.problem = 'rarm_problem';
CCFG.datadir = pwd;
CCFG.gamma = 0.98;
CCFG.eps = 0;
CCFG.maxiter = 50;
CCFG.sampledisp = .01;
CCFG.itersave = 10;
CCFG = setfigprop(CCFG, 'addfields');
ccfg = parseconfig(c, CCFG);

% common online config
OCFG = ccfg;
OCFG.trialtime = 10;                        % seconds per trial
OCFG.dupd = 50 * 0.05;                      % update each this # of samples
OCFG.lspi_params.maxtime = 60*120;          % run for 2 simulated hours
OCFG.explordecay = 0.001^(1/OCFG.lspi_params.maxtime);
OCFG.lspi_params.dstoretheta = OCFG.dupd;   % store all thetas
OCFG.lspi_params.ddisp = 5;                 % display stats after this #seconds
OCFG.lspi_params.dsave = 120;               % save once this #seconds
% output config
OCFG.startat = 1; OCFG.repeat = 10;
OCFG.show = {'rep'};
OCFG.donlineperf = [40:60:1000 1080:180:OCFG.lspi_params.maxtime]; 
OCFG.X0 = 'pos'; 
OCFG.mc_eps = 0.1;
OCFG.plotminmax = 0; OCFG.plotconf = 0;
OCFG.transparency = 1;
OCFG.minmaxtype = 'line';
OCFG.xlim = [0 OCFG.lspi_params.maxtime];
OCFG.type = 'rbfdisc';
OCFG.N = 5; OCFG.xspace = 'eq';
OCFG.M = 3; OCFG.uspace = 'eq';
OCFG.testalpha = [.001 .01 .1 .5 1];        % values of alpha to test

ocfg = parseconfig(c, OCFG);

% ==== Online LSPI
if checkrunexp('lspio', exp), 
    lspiocfg = ocfg; 
    lspiocfg.alg = 'lspionline';
    lspiexp(lspiocfg); 
end;

% % ==== Online LSPE tryout, preliminary experiment to choose alpha (stepsize)
% % The code below does NOT work in 3.X GB of RAM -- out of memory errors...
% % So unfortunately LSPE cannot be tested in this configuration
% if checkrunexp('lspeoca', exp), 
%     lspeocfg = ocfg; 
%     lspeocfg.alg = 'lspeonline';
%     lspeocfg.repeat = 1;
%     lspeocfg.miscinfo = 'choosealpha';
%     lspeocfg.infix = 'choosealpha';
% 	lspeocfg.alpha = ocfg.testalpha;
%     lspeocfg.show = {'alpha'};     % don't show repetition number, it is useless
%     % use the same exploration steps (although they will not be in the same states...)
%     lspeocfg.lspi_params.randinit = {'twister', 1234567890};
%     lspiexp(lspeocfg); 
% end;
