function ip_Tsstudy

% study the influence of the sample time on the solution (sampling artifacts:)
% cfg.Ts = [0.1 0.075 0.05 0.04 0.03 0.02 0.01 0.005];
cfg.Ts = [0.005 0.0075 0.01 0.015 0.02 0.025 0.03 0.04 0.05 0.1]; % coarser version
% cfg.Ts = [0.005 0.01 0.03 0.05 0.1];  % used to send pictures to Robert for Jelmer's ACO
% application
cfg.gamma = 0.98;       % single gamma, or reference gamma for the first sample time
cfg.adaptivegamma = 1;
cfg.loadapproxgamma =[];

% what to output
cfg.fzh = 1;
cfg.fzv = 0;
cfg.fzreplay = 0;
cfg.lspisol = 0;
cfg.lspireplay = 0;
cfg.plottarget = 'screen';
cfg.savedir = 'D:\Work\m\approxrl\systems\invpend_setup\Tsstudy\';

% repeat gamma or Ts one according to the other to get the experiment settings
if isscalar(cfg.gamma) && ~isscalar(cfg.Ts),
    if cfg.adaptivegamma,
        % change gamma values such that the time horizon remains the same as for the first gamma/Ts
        % combination
        alpha = cfg.gamma ^ (1/cfg.Ts(1));
        cfg.gamma = alpha .^ cfg.Ts;
    else
        cfg.gamma = cfg.gamma + 0*cfg.Ts;
    end;
end;
if isscalar(cfg.Ts) && ~isscalar(cfg.gamma),
    cfg.Ts = cfg.Ts + 0*cfg.gamma;
end;

% echo config
cfg

for i = 1:length(cfg.Ts),
    % ========================
    % run fuzzy Q-iteration
    df = gendfname('fuzzyqi', cfg.Ts(i), cfg.gamma(i), 'u2');
    if (cfg.fzh || cfg.fzreplay) && ~exist([df '.mat'], 'file'),     % need to run experiment
        rcfg = struct;
        rcfg.run = 1;
        rcfg.gamma = cfg.gamma(i);
        rcfg.eps = 1e-2; 
        rcfg.maxiter = 3000;
        rcfg.problem = 'ipsetup_problem';
        rcfg.datafile = df;
        rcfg.fuzzy_params = {'mid'};
        rcfg.model_params = {'lqr', [], struct('Ts', cfg.Ts(i))};
%         rcfg.model_params = {'lqr', 'rot', struct('Ts', cfg.Ts(i))};
        if ~isempty(cfg.loadapproxgamma),
            rcfg.loadapprox = gendfname('fuzzyqi', cfg.Ts(i), cfg.loadapproxgamma);
        end; 
        fuzzyqi(rcfg);
    end;

    % plot fuzzy policy
    if cfg.fzh,
        pcfg = struct; pcfg.fuzzyh = 1; pcfg.datafile = df;
        pcfg = copyfields(cfg, pcfg, {'plottarget', 'savedir'}); pcfg.savefig = [df '_h'];
        dc_plot(pcfg);
    end;
    % plot VF
    if cfg.fzv,
        pcfg = struct; pcfg.fuzzyv = 1; pcfg.datafile = df;
        pcfg = copyfields(cfg, pcfg, {'plottarget', 'savedir'}); pcfg.savefig = [df '_v'];
        dc_plot(pcfg);
    end;

    % replay from stable eq.
    if cfg.fzreplay,
        recfg = struct;
        recfg.replay = 1;
        recfg.problem = 'ipsetup_problem';
        recfg.tend = 5;
        recfg.datafile = df;
        recfg.x0 = [pi, 0];
        recfg.interph = 0;
        fuzzyqi(recfg);
    end;


end;    % FOR experiments

end         % FUNCTION ip_Tsstudy()


% -------------------
% Local functions

% Generate data file name
function df = gendfname(alg, Ts, gamma, infix, acfg, scfg)
% acfg, scfg only need to be specified for lspi
    if nargin < 4, infix = ''; end;
    if nargin < 5, acfg = []; scfg = []; end;
    switch alg,
        case 'fuzzyqi',
            df = 'fzips_mid';
            if ~isempty(infix), df = [df '_' infix]; end;
    end;
    % gamma and Ts are common
    str = num2str(gamma); str = str(str ~= '.');
    df = [df '_g' str];
    str = num2str(Ts); str = str(str ~= '.');
    df = [df '_Ts' str];
end
